#define APPNAME "PlugView"

/******************************************************************************\
    PROGRAM:    PlugView.C
    Author:     Robert B. Hess
    Date:       1-Jun-95
\******************************************************************************/
// This is a sample application to illustrate how an application might watch
// for the various 'Plug And Play' messages that are available on Windows 95.
// It is important to note, that there may be some differences between how
// Windows 95 is 'currently' broadcasting Plug And Play messages, and how
// these messages are implemented on Windows NT, and future versions of
// Windows 9x. Please be careful when monitoring Plug And Play events, every
// attempt was made to make sure that the sample code listed here will run on
// Windows NT when it supports Plug And Play.

/*
 *    (C) Copyright Microsoft Corp. 1993.  All rights reserved.
 *
 *    You have a royalty-free right to use, modify, reproduce and 
 *    distribute the Sample Files (and/or any modified version) in 
 *    any way you find useful, provided that you agree that 
 *    Microsoft has no warranty obligations or liability for any 
 *    Sample Application Files which are modified. 
 */

// Windows Header Files:
#include <windows.h>
#include <dbt.h>	// For WM_DEVICECHANGE et. al.
#include <pbt.h>	// For WM_POWERBROADCAST et. al.
#include <commctrl.h> // For the 'RichEdit' control
#include <richedit.h> // For the 'RichEdit' control

// C RunTime Header Files
#include <stdlib.h>
#include <malloc.h>
#include <memory.h>

// Local Header Files
#include "PlugView.h"

// Just to make it easier to determine appropriate code paths:
#if defined (WIN32) // This had better be true, but just in case...
	#define IS_WIN32 TRUE
#else
	#define IS_WIN32 FALSE
#endif
#define IS_WINNT   IS_WIN32 && (BOOL)(GetVersion() < 0x80000000)
#define IS_WIN32S  IS_WIN32 && (BOOL)(!(IS_WINNT) && (LOBYTE(LOWORD(GetVersion()))<4))
#define IS_WIN95   (BOOL)(!(IS_WINNT) && !(IS_WIN32S)) && IS_WIN32

/////////////////////////////////////////////////////////////////////////
// Global Variables:

HINSTANCE hInst; // current instance
char szAppName[] = APPNAME; // The name of this application
char szTitle[]   = APPNAME": Plug And Play Event Monitor"; // The title bar text

/////////////////////////////////////////////////////////////////////////
// Foward declarations of functions included in this code module:

BOOL InitApplication(HINSTANCE);
BOOL InitInstance(HINSTANCE, int);
LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);
BOOL CenterWindow (HWND, HWND);
LRESULT CALLBACK About(HWND, UINT, WPARAM, LPARAM);
HICON LoadSmallIcon (HINSTANCE hInstance, LPCTSTR lpszIcon);
LRESULT CALLBACK PlugList(HWND, UINT, WPARAM, LPARAM);

/************************************************************************\
 *    FUNCTION: WinMain
\************************************************************************/

int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	MSG msg;
	HANDLE hAccelTable;
	HANDLE hRTFLib = NULL; // NEEDED FOR RICHEDIT!

	hRTFLib = LoadLibrary("RICHED32.DLL");
	if (!hRTFLib) MessageBox (GetFocus(), "Failed to locate RICHED32.DLL", szAppName, 0);

	if (!hPrevInstance) {
		// Perform instance initialization:
		if (!InitApplication(hInstance)) {
			return (FALSE);
		}
	}

	// Perform application initialization:
	if (!InitInstance(hInstance, nCmdShow)) {
		return (FALSE);
	}

	hAccelTable = LoadAccelerators (hInstance, szAppName);

	// Main message loop:
	while (GetMessage(&msg, NULL, 0, 0)) {
		if (!TranslateAccelerator (msg.hwnd, hAccelTable, &msg)) {
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}

	if (hRTFLib) FreeLibrary (hRTFLib);

	return (msg.wParam);

	lpCmdLine; // This will prevent 'unused formal parameter' warnings
}


/************************************************************************\
 *    FUNCTION: InitApplication
\************************************************************************/

BOOL InitApplication(HINSTANCE hInstance)
{
	WNDCLASS  wc;
    HWND      hwnd;

    // Win32 will always set hPrevInstance to NULL, so lets check
    // things a little closer. This is because we only want a single
	// version of this app to run at a time
    hwnd = FindWindow (szAppName, NULL);
    if (hwnd) {
		// We found another version of ourself. Lets defer to it:
        if (IsIconic(hwnd)) {
            ShowWindow(hwnd, SW_RESTORE);
        }
        SetForegroundWindow (hwnd);
		// If this app actually had any functionality, we would
		// also want to communicate any action that our 'twin'
		// should now perform based on how the user tried to
		// execute us.
        return FALSE;
    }

	// Fill in window class structure with parameters that describe
	// the main window.
	wc.style         = CS_HREDRAW | CS_VREDRAW;
	wc.lpfnWndProc   = (WNDPROC)WndProc;
	wc.cbClsExtra    = 0;
	wc.cbWndExtra    = 0;
	wc.hInstance     = hInstance;
	wc.hIcon         = LoadIcon (hInstance, szAppName);
	wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
	wc.lpszMenuName  = szAppName;
	wc.lpszClassName = szAppName;

	// Register the main window class
	if (!RegisterClass(&wc)) {
		return FALSE;
	}

	return TRUE;
}


/************************************************************************\
 *    FUNCTION: InitInstance
\************************************************************************/

BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{
	HWND hWnd;
	HICON hIconSm;
	
	hInst = hInstance; // Store instance handle in our global variable

	hWnd = CreateWindow(szAppName, szTitle, WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN,
		CW_USEDEFAULT, 0, CW_USEDEFAULT, 0,
		NULL, NULL, hInstance, NULL);

	if (!hWnd) {
		return (FALSE);
	}

// In case we are compiling this on a Windows NT development system, we
// will need to add a definition for WM_SETICON.
#if !defined(WM_SETICON)
#define WM_SETICON 0x0080
#endif
	hIconSm = LoadSmallIcon(hInstance, szAppName);
	if (hIconSm) {
		SendMessage (hWnd, WM_SETICON, FALSE, (LPARAM)hIconSm);
	}

	ShowWindow(hWnd, nCmdShow);
	UpdateWindow(hWnd);

	return (TRUE);
}


/************************************************************************\
 *    FUNCTION: WndProc
\************************************************************************/

HWND hwndOutput = NULL;

LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	int wmId, wmEvent;
	PAINTSTRUCT ps;
	HDC hdc;
    POINT pnt;
    HMENU hMenu;
    BOOL bGotHelp;
	RECT rClient;

	switch (message) { 

		case WM_CREATE:
			InitCommonControls();
			SetLastError(0);

    		hwndOutput = CreateWindow("RichEdit",
				"-- Plug And Play Notifications ---------\r\n", 
				WS_CHILD | WS_CLIPSIBLINGS | WS_VISIBLE
				| WS_HSCROLL | WS_VSCROLL
				| ES_READONLY 
				| ES_MULTILINE | ES_WANTRETURN // | ES_AUTOVSCROLL | ES_AUTOHSCROLL,
				| ES_SAVESEL | ES_NOHIDESEL
				| ES_SELECTIONBAR | ES_SUNKEN,
				0,0,100,100, // NOTE: RICHEDIT controls require a 'valid' size
				hWnd, (HMENU)ID_OUTPUT, hInst, NULL);

			if (!hwndOutput) {
				// just in case we failed on the RichEdit, fall back to a normal
				// edit control...
				DWORD dwErr = GetLastError();
				char szMsg[80];
				wsprintf (szMsg, "Unable to create RICHEDIT control : 0x%X : %d", dwErr, dwErr);
				MessageBox (GetFocus(), szMsg, "PlugView", 0);
				hwndOutput = CreateWindow ("EDIT", "-- Plug And Play Notifications ---------\r\n", 
					WS_CHILD | WS_CLIPSIBLINGS | WS_VISIBLE
					| WS_HSCROLL | WS_VSCROLL
					| ES_READONLY 
					| ES_MULTILINE | ES_WANTRETURN,// | ES_AUTOVSCROLL | ES_AUTOHSCROLL,
					0,0,0,0,
					hWnd, (HMENU)ID_OUTPUT, hInst, NULL);
			}

			if (hwndOutput) {
				SendMessage (hwndOutput, WM_SETFONT, (WPARAM)GetStockObject(ANSI_VAR_FONT), 0);
			} else {
				MessageBox (GetFocus(), "Unable to create EDIT control", "PlugView", 0);
			}
			break;

		case WM_CTLCOLORSTATIC:
		case WM_CTLCOLOREDIT:
			{
				HBRUSH hbr;
				HDC hdc;

				hdc = (HDC)wParam;
				hbr = (HBRUSH)GetStockObject (BLACK_BRUSH);
				SetBkColor (hdc, 0);
				SetTextColor (hdc, RGB(0,255,0));
				return (LRESULT)(HBRUSH)hbr;
			}

		case WM_SIZE:
			if (IsWindow(hwndOutput)) {
				GetClientRect (hWnd, &rClient);
				SetWindowPos  (hwndOutput, NULL, 
					rClient.left, rClient.top, rClient.right-rClient.left, rClient.bottom-rClient.top,
					SWP_NOZORDER);
			} else {
				MessageBox(GetFocus(), "Invalid output window", "PlugView", 0);
			}
			break;

		case WM_INITMENUPOPUP:
			{
				HMENU hmenuPopup = (HMENU) wParam;			// handle of pop-up menu  
				UINT uPos = (UINT) LOWORD(lParam);			// pop-up item position   
				BOOL fSystemMenu = (BOOL) HIWORD(lParam);	// System menu flag
				int id;
				DWORD dwSel;
				if (!fSystemMenu) {
					// Cheap way to determine which menu we are on is to grab the 'zeroth'
					// item from the menu, and switch on that. If the menu ever changes,
					// these items must be changed to reflect that current 'zeroth' items:
					id = GetMenuItemID (hmenuPopup, 0);
					if (id == IDM_UNDO) { // "Edit" menu
						dwSel = SendMessage (hwndOutput, EM_GETSEL, (LPARAM)NULL, (WPARAM)NULL);
						if (HIWORD(dwSel)-LOWORD(dwSel)>0) {
							EnableMenuItem(hmenuPopup, IDM_COPY, MF_ENABLED);
						} else {
							EnableMenuItem(hmenuPopup, IDM_COPY, MF_GRAYED);
						}
						break;
					}
				}
			}
			break;

		case WM_COMMAND:
			wmId    = LOWORD(wParam); // Remember, these are...
			wmEvent = HIWORD(wParam); // ...different for Win32!

			// Deal with the menu selections:
			switch (wmId) {

				case IDM_ABOUT:
					// Make and FreeProcInstance aren't needed on Win32!
					//lpProcAbout = MakeProcInstance((FARPROC)About, hInst);
                    DialogBox(hInst, "AboutBox", hWnd, (DLGPROC)About);
					//FreeProcInstance(lpProcAbout);
					break;

				case IDM_EXIT:
					DestroyWindow (hWnd);
					break;

                case IDM_HELPTOPICS: // Only called on Windows 95
// In case we are compiling this on a Windows NT development system, we
// will need to add a definition for HELP_FINDER.
#if !defined(HELP_FINDER)
#define HELP_FINDER 0x000b
#endif
                    bGotHelp = WinHelp (hWnd, APPNAME".HLP", HELP_FINDER,(DWORD)0);
                    if (!bGotHelp) {
                        MessageBox (GetFocus(),"Unable to activate help",
                            szAppName, MB_OK|MB_ICONHAND);
					}
					break;

				case IDM_HELPCONTENTS: // Not called on Windows 95
                    bGotHelp = WinHelp (hWnd, APPNAME".HLP", HELP_CONTENTS,(DWORD)0);
                    if (!bGotHelp) {
                        MessageBox (GetFocus(),"Unable to activate help",
                            szAppName, MB_OK|MB_ICONHAND);
					}
					break;

				case IDM_HELPSEARCH: // Not called on Windows 95
                    if (!WinHelp(hWnd, APPNAME".HLP", HELP_PARTIALKEY, (DWORD)(LPSTR)"")) {
						MessageBox (GetFocus(), "Unable to activate help",
							szAppName, MB_OK|MB_ICONHAND);
					}
					break;

				case IDM_HELPHELP: // Not called on Windows 95
					if(!WinHelp(hWnd, (LPSTR)NULL, HELP_HELPONHELP, 0)) {
						MessageBox (GetFocus(), "Unable to activate help",
							szAppName, MB_OK|MB_ICONHAND);
					}
					break;

				case IDM_COPY:
					SendMessage (hwndOutput, WM_COPY, 0, 0);
					break;

				// Here are all the other possible menu options,
				// all of these are currently disabled:
				case IDM_NEW:
				case IDM_OPEN:
				case IDM_SAVE:
				case IDM_SAVEAS:
				case IDM_UNDO:
				case IDM_CUT:
				case IDM_PASTE:
				case IDM_LINK:
				case IDM_LINKS:

				default:
					return (DefWindowProc(hWnd, message, wParam, lParam));
			}
			break;

        case WM_LBUTTONDOWN:
			{
				// This would be one good place to put 'test' code. Thus
				// when you click the left button in the client area,
				// it would fire off the test code that you are wanting
				// to check out.
			}
            break;

        case WM_NCRBUTTONUP: // RightClick on windows non-client area...
			if (IS_WIN95 && SendMessage(hWnd, WM_NCHITTEST, 0, lParam) == HTSYSMENU) {
				// The user has clicked the right button on the applications
				// 'System Menu'. Here is where you would alter the default
				// system menu to reflect your application. Notice how the
				// explorer deals with this. For this app, we aren't doing
				// anything
				return (DefWindowProc(hWnd, message, wParam, lParam));
			} else {
				// Nothing we are interested in, allow default handling...
				return (DefWindowProc(hWnd, message, wParam, lParam));
			}
            break;

        case WM_RBUTTONDOWN: // RightClick in windows client area...
            pnt.x = LOWORD(lParam);
            pnt.y = HIWORD(lParam);
            ClientToScreen(hWnd, (LPPOINT) &pnt);

			// This is where you would determine the appropriate 'context'
			// menu to bring up. Since this app has no real functionality,
			// we will just bring up the 'Help' menu:
            hMenu = GetSubMenu (GetMenu (hWnd), 2);
            if (hMenu) {
                TrackPopupMenu (hMenu, 0, pnt.x, pnt.y, 0, hWnd, NULL);
            } else {
				// Couldn't find the menu...
                MessageBeep(0);
            }
            break;

		// Main Plug&Play Messages
		case WM_DEVICECHANGE:
		case WM_POWERBROADCAST:
		case WM_DISPLAYCHANGE:

		// Some additional messages new to Windows 95
		case WM_INPUTLANGCHANGEREQUEST:
		case WM_INPUTLANGCHANGE:
		case WM_USERCHANGED:

		// Some additional messages
		case WM_SETTINGCHANGE:
		case WM_POWER:
		case WM_DEVMODECHANGE:

		// And lets not forget:
		case WM_QUERYENDSESSION:
		case WM_ENDSESSION:

			PlugList (hWnd, message, wParam, lParam);
			return (DefWindowProc(hWnd, message, wParam, lParam));

		case WM_PAINT:
			hdc = BeginPaint (hWnd, &ps);
			// Add any drawing code here...
			EndPaint (hWnd, &ps);
			break;        

		case WM_DESTROY:
			// Tell WinHelp we don't need it any more...
            WinHelp (hWnd, APPNAME".HLP", HELP_QUIT,(DWORD)0);
			PostQuitMessage(0);
			break;

		default:
			return (DefWindowProc(hWnd, message, wParam, lParam));
	}
	return (0);
}


//////////////////////////////////////////////////////////////////////////
//
// Extra Credit:
//
// The following routines are just the 'something extra' that programs
// should do in order to give the user a good experience.
//
//////////////////////////////////////////////////////////////////////////


/************************************************************************\
 *    FUNCTION: CenterWindow
\************************************************************************/
// This is a 'utility' function I find usefull. It will center one
// window over another. It also makes sure that the placement is within
// the 'working area', meaning that it is both within the display limits
// of the screen, -and- not obscured by the tray or other frameing
// elements of the desktop.
BOOL CenterWindow (HWND hwndChild, HWND hwndParent)
{
	RECT    rChild, rParent, rWorkArea = {0,0,0,0};
	int     wChild, hChild, wParent, hParent;
	int     xNew, yNew;
	BOOL	bResult;

	// Get the Height and Width of the child window
	GetWindowRect (hwndChild, &rChild);
	wChild = rChild.right - rChild.left;
	hChild = rChild.bottom - rChild.top;

	// Get the Height and Width of the parent window
	GetWindowRect (hwndParent, &rParent);
	wParent = rParent.right - rParent.left;
	hParent = rParent.bottom - rParent.top;

	// Get the limits of the 'workarea'
// In case we are compiling this on a Windows NT development system, we
// will need to add a definition for SPI_GETWORKAREA. If we try using
// this on Windows NT, it will properly return back a 'failure'.
#if !defined(SPI_GETWORKAREA)
#define SPI_GETWORKAREA 48
#endif
	bResult = SystemParametersInfo( SPI_GETWORKAREA, sizeof(RECT), &rWorkArea, 0);
	if (!bResult) {
		rWorkArea.left = rWorkArea.top = 0;
		rWorkArea.right = GetSystemMetrics(SM_CXSCREEN);
		rWorkArea.bottom = GetSystemMetrics(SM_CYSCREEN);
	}

	// Calculate new X position, then adjust for workarea
	xNew = rParent.left + ((wParent - wChild) /2);
	if (xNew < rWorkArea.left) {
		xNew = rWorkArea.left;
	} else if ((xNew+wChild) > rWorkArea.right) {
		xNew = rWorkArea.right - wChild;
	}

	// Calculate new Y position, then adjust for workarea
	yNew = rParent.top  + ((hParent - hChild) /2);
	if (yNew < rWorkArea.top) {
		yNew = rWorkArea.top;
	} else if ((yNew+hChild) > rWorkArea.bottom) {
		yNew = rWorkArea.bottom - hChild;
	}

	// Set it, and return
	return SetWindowPos (hwndChild, NULL, xNew, yNew, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
}


/************************************************************************\
 *    FUNCTION: About
\************************************************************************/
// To allow us better flexibility over the contents of the 'About' box,
// lets go the extra mile and pull out values from the 'Version' resource.
// This means that by properly keeping that information up-to-date, it
// will automatically keep your about box information current. Remember,
// that the 'version' strings can also be seen from the 'Properties'
// page of FileManager on Windows NT, or the Explorer on Windows 95.

LRESULT CALLBACK About(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	static  HFONT hfontDlg;		// Font for dialog text
	static	HFONT hFinePrint;	// Font for 'fine print' in dialog

	DWORD   dwVerInfoSize;		// Size of version information block
	LPSTR   lpVersion;			// String pointer to 'version' text
	DWORD   dwVerHnd=0;			// An 'ignored' parameter, always '0'
	UINT    uVersionLen;		// Current length of full version string
	WORD    wRootLen;			// length of the 'root' portion of string
	BOOL    bRetCode;			// Boolean returned value
	int     i;					// for the for loop
	char    szFullPath[256];	// full path of module
	char    szResult[256];		// Temporary result string
	char    szGetName[256];		// String to use for extracting version info
	DWORD	dwVersion;			// To hold return value from GetVersion
	char	szVersion[40];		// Temporary string for building output
	DWORD	dwResult;			// DWORD return value

	switch (message) {
		// When the dialog is first being initialized, we want to do a number
		// of things to 'customize' the look of this 'About' box.
		// 1. Substitute a 'non-bold' font for most of the text
		// 2. Substitute a 'fine print' font for the legal text at the
		//    bottom.
		// 3. Center the dialog over the parent window, and keep it within
		//    the limits of the screen.
		// 4. Substitute strings from the 'version' section of the resource
		//    for various strings in the dialog
		// 5. Display information regarding the 'version' of Windows that
		//    we are running on
		//
        case WM_INITDIALOG:
			// Create some replacement fonts:
			hfontDlg = GetStockObject (ANSI_VAR_FONT);
			hFinePrint = CreateFont(11, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
				VARIABLE_PITCH | FF_SWISS, "");

			// Reposition the dialog:
			CenterWindow (hDlg, GetWindow (hDlg, GW_OWNER));

			// Now lets dive in and pull out the version information:
			GetModuleFileName (hInst, szFullPath, sizeof(szFullPath));
			dwVerInfoSize = GetFileVersionInfoSize(szFullPath, &dwVerHnd);
			if (dwVerInfoSize) {
				LPSTR   lpstrVffInfo;
				HANDLE  hMem;
				hMem = GlobalAlloc(GMEM_MOVEABLE, dwVerInfoSize);
				lpstrVffInfo  = GlobalLock(hMem);
				GetFileVersionInfo(szFullPath, dwVerHnd, dwVerInfoSize, lpstrVffInfo);
				// The below 'hex' value looks a little confusing, but
				// essentially what it is, is the hexidecimal representation
				// of a couple different values that represent the language
				// and character set that we are wanting string values for.
				// 040904E4 is a very common one, because it means:
				//   US English, Windows MultiLingual characterset
				// Or to pull it all apart:
				// 04------        = SUBLANG_ENGLISH_USA
				// --09----        = LANG_ENGLISH
				// ----04E4 = 1252 = Codepage for Windows:Multilingual
				lstrcpy(szGetName, "\\StringFileInfo\\040904E4\\");	 
				wRootLen = lstrlen(szGetName); // Save this position
			
				// Set the title of the dialog:
				lstrcat (szGetName, "ProductName");
				bRetCode = VerQueryValue((LPVOID)lpstrVffInfo,
					(LPSTR)szGetName,
					(LPVOID)&lpVersion,
					(UINT *)&uVersionLen);
				lstrcpy(szResult, "About ");
				lstrcat(szResult, lpVersion);
				SetWindowText (hDlg, szResult);

				// Walk through the dialog items that we want to replace:
				for (i = DLG_VERFIRST; i <= DLG_VERLAST; i++) {
					GetDlgItemText(hDlg, i, szResult, sizeof(szResult));
					szGetName[wRootLen] = (char)0;
					lstrcat (szGetName, szResult);
					uVersionLen   = 0;
					lpVersion     = NULL;
					bRetCode      =  VerQueryValue((LPVOID)lpstrVffInfo,
						(LPSTR)szGetName,
						(LPVOID)&lpVersion,
						(UINT *)&uVersionLen);

					if ( bRetCode && uVersionLen && lpVersion) {
					// Replace dialog item text with version info
						lstrcpy(szResult, lpVersion);
						SetDlgItemText(hDlg, i, szResult);
					} else {
						dwResult = GetLastError();
						wsprintf (szResult, "Error %lu", dwResult);
						SetDlgItemText (hDlg, i, szResult);
					}
					SendMessage (GetDlgItem (hDlg, i), WM_SETFONT, 
						(UINT)((i==DLG_VERLAST)?hFinePrint:hfontDlg),
						TRUE);
				} // for (i = DLG_VERFIRST; i <= DLG_VERLAST; i++)


				GlobalUnlock(hMem);
				GlobalFree(hMem);

			} else {
				// No version information available.
			} // if (dwVerInfoSize)

            SendMessage (GetDlgItem (hDlg, IDC_LABEL), WM_SETFONT,
				(WPARAM)hfontDlg,(LPARAM)TRUE);

			// Get the version information about Windows:
			// We are  using GetVersion rather then GetVersionEx
			// because earlier versions of Windows NT and Win32s
			// didn't include GetVersionEx:
			dwVersion = GetVersion();

			if (dwVersion < 0x80000000) {
				// Windows NT
				wsprintf (szVersion, "Microsoft Windows NT %u.%u (Build: %u)",
					(DWORD)(LOBYTE(LOWORD(dwVersion))),
					(DWORD)(HIBYTE(LOWORD(dwVersion))),
                    (DWORD)(HIWORD(dwVersion)) );
			} else if (LOBYTE(LOWORD(dwVersion))<4) {
				// Win32s
                wsprintf (szVersion, "Microsoft Win32s %u.%u (Build: %u)",
					(DWORD)(LOBYTE(LOWORD(dwVersion))),
					(DWORD)(HIBYTE(LOWORD(dwVersion))),
                    (DWORD)(HIWORD(dwVersion) & ~0x8000) );
			} else {
				// Windows 95
                wsprintf (szVersion, "Microsoft Windows 95  (osv: %u.%u)",
                    (DWORD)(LOBYTE(LOWORD(dwVersion))),
                    (DWORD)(HIBYTE(LOWORD(dwVersion))) );
			}

		    SetWindowText (GetDlgItem(hDlg, IDC_OSVERSION), szVersion);
			return (TRUE);

		case WM_COMMAND:
			// Only possible option is to close down. Make sure we
			// clean things up on our way out...
			if (LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL) {
				EndDialog(hDlg, TRUE);
				//DeleteObject (hfontDlg); // don't delete stock objects
				DeleteObject (hFinePrint);
				return (TRUE);
			}
			break;
	}

    return FALSE;
}

/************************************************************************\
 *    FUNCTION: LoadSmallIcon
\************************************************************************/
HICON LoadSmallIcon (HINSTANCE hInstance, LPCTSTR lpszIcon)
{
	HMODULE  hMod;

	typedef int (__stdcall *PROCPTR)(HINSTANCE, LPCSTR, UINT, int, int, UINT);
	PROCPTR proc;

	hMod = GetModuleHandle ("USER32");
	if (hMod != NULL) {

#if defined (UNICODE)
		proc = (PROCPTR)GetProcAddress (hMod, "LoadImageW");
#else
		proc = (PROCPTR)GetProcAddress (hMod, "LoadImageA");
#endif

		if (proc != NULL) {
			//return LoadImage(hInstance, lpszIcon, IMAGE_ICON, 16, 16, 0);
#if !defined(IMAGE_ICON)
#define IMAGE_ICON 1
#endif
			return (HICON)(*proc)(hInstance, lpszIcon, IMAGE_ICON, 16, 16, 0);//return 
		}
	}
	return NULL;
}

// ===========================================================================
// SendString (HWND, LPSTR)
// ---------------------------------------------------------------------------
// Adds the specified string to the text being displayed in the edit control
// (either normal edit control or 'richedit' control) specified by the
// HWND parameter.

void SendString (HWND hwnd, LPSTR sz)
{
	int iLen, iStart, iEnd;

	SendMessage(hwnd, EM_GETSEL, (WPARAM)&iStart, (LPARAM)&iEnd);
	iLen = GetWindowTextLength (hwnd);
	SendMessage (hwnd, EM_SETSEL, iLen, iLen);
	SendMessage (hwnd, EM_REPLACESEL, 0, (LPARAM)sz);
	iLen = GetWindowTextLength (hwnd);
	SendMessage (hwnd, EM_SETSEL, iLen, iLen);
	if (iEnd-iStart>0) {
		SendMessage(hwnd, EM_SETSEL, (WPARAM)iStart, (LPARAM)iEnd);
	}
	SendMessage (hwnd, EM_SCROLLCARET, 0, 0);
}

// ===========================================================================
// AppendEditString (HWND, LPSTR)
// ---------------------------------------------------------------------------
// Appends the specified string to the current string buffer being worked on.
// When the string buffer is full, or when a 'carriage return' is encountered,
// the buffer will be sent to the edit control specified by the HWND
// parameter. Building the string buffer this way, instead of sending each
// fragment to the window, greatly increases the speed.

void AppendEditString (HWND hwnd, LPSTR sz)
{
	static char szBuf[256];
	static int iPos = 0;
	char *pChar = sz;

	while (*pChar) {
		if (*pChar == '\n') {
			SendString (hwnd, szBuf);
			SendString (hwnd, "\r\n");
			iPos = 0;
		} else {
			szBuf[iPos++] = *pChar;
			if (iPos >= 253) {
				szBuf[iPos] = (char)0;
				SendString (hwnd, szBuf);
				iPos = 0;
			}
		}
		szBuf[iPos] = (char)0;
		pChar++;
	}
}

// ===========================================================================
// PlugList (HWND, UINT, WPARAM, LPARAM)
// ---------------------------------------------------------------------------
// This functions deals with virtually all of the messages that could be
// considered of the 'plug and play' variety. This includes not only the
// 'new' messages for Windows 95 that are specifically Plug And Play
// (WM_DEVICECHANGE, WM_POWERBROADCAST, WM_DISPLAYCHANGE), but also the other
// messages that already existed in the system, and denoted potential system
// wide changes taking place that a 'Plug And Play' aware application might
// want to be monitoring.
LRESULT CALLBACK PlugList(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	char sz[128];
	DEV_BROADCAST_HDR *pDBH;
	struct _DEV_BROADCAST_USERDEFINED *pDBU;
	SYSTEMTIME st;
	
	// Display the current time. This is not the time the message was posted,
	// but it should be close enough for our purposes.	
	GetLocalTime(&st);
	wsprintf (sz, "[%02.2d:%02.2d:%02.2d] ",
		(int)st.wHour, (int)st.wMinute, (int)st.wSecond);

	// AppendEditString is a function included in this code module for pushing
	// text out to our client window, which happens to be a RichEdit control
	AppendEditString (hwndOutput, sz);

	switch (message) {
		case WM_DEVICECHANGE:
		// The WM_DEVICECHANGE message notifies an application or device driver
		// of a change to the hardware configuration of a device or the computer.
		// The events available are currently:
		//
		//	DBT_DEVICEARRIVAL			A device has been inserted and is now available. 
		//	DBT_DEVICEQUERYREMOVE		Permission to remove a device is requested.
		//	DBT_DEVICEQUERYREMOVEFAILED	Request to remove a device has been canceled.
		//	DBT_DEVICEREMOVEPENDING		Device is about to be removed. Can not be denied.
		//	DBT_DEVICEREMOVECOMPLETE	Device has been removed.
		//	DBT_DEVICETYPESPECIFIC		Device-specific event.
		//	DBT_CONFIGCHANGED			Current configuration has changed.
			AppendEditString (hwndOutput, "WM_DEVICECHANGE");
			{
				UINT uEvent = (UINT) wParam;	// The event type
				DWORD dwData = (DWORD) lParam;	// Pointer to Event-specific data structure

				AppendEditString (hwndOutput, " : ");

				// List the 'type' of message that has been recieved:
				switch (uEvent) {

					// DBT_DEVICE* messages indicate changes in a device state
					// have occured.
					// The lParam of these events points to a DEV_BROADCAST_HDR
					// or one of its derivatives:
					case DBT_DEVICEARRIVAL:	//A device has been inserted and is now available. 
						AppendEditString (hwndOutput, "DBT_DEVICEARRIVAL");
						break;
					case DBT_DEVICEQUERYREMOVE:	//Permission to remove a device is requested.
						// Any application can deny this request and cancel the removal.
						AppendEditString (hwndOutput, "DBT_DEVICEQUERYREMOVE");
						break;
					case DBT_DEVICEQUERYREMOVEFAILED:	//Request to remove a device has been canceled.
						AppendEditString (hwndOutput, "DBT_DEVICEQUERYREMOVEFAILED");
						break;
					case DBT_DEVICEREMOVEPENDING:	//Device is about to be removed. Can not be denied.
						AppendEditString (hwndOutput, "DBT_DEVICEREMOVEPENDING");
						break;
					case DBT_DEVICEREMOVECOMPLETE:	//Device has been removed.
						AppendEditString (hwndOutput, "DBT_DEVICEREMOVECOMPLETE");
						break;
					case DBT_DEVICETYPESPECIFIC:	//Device-specific event.
						AppendEditString (hwndOutput, "DBT_DEVICETYPESPECIFIC");
						break;
					case DBT_CONFIGCHANGED:	//Current configuration has changed.
						// This usually only occures on Dock and Undock.
						AppendEditString (hwndOutput, "DBT_CONFIGCHANGED");
						break;

					// DBT_USERDEFINED: identifies a user-defined system message.
					// lParam points to a _DEV_BROADCAST_USERDEFINED structure.
					case DBT_USERDEFINED:
						AppendEditString (hwndOutput, "DBT_USERDEFINED");
						break;

					// DBT_MONITORCHANGE: this message is sent when the display monitor has changed
					// lParam indicates  new resolution to use (LOWORD=x, HIWORD=y)
					// if 0, use the default res for current config
					case DBT_MONITORCHANGE:
						AppendEditString (hwndOutput, "DBT_MONITORCHANGE");
						break;

					// The following don't use lParam:			
					case DBT_QUERYCHANGECONFIG:
						// Sent to ask if a configuration change is allowed
						AppendEditString (hwndOutput, "DBT_QUERYCHANGECONFIG");
						break;
					case DBT_CONFIGCHANGECANCELED:
						// Sent if somebody rejected the configuration change
						AppendEditString (hwndOutput, "DBT_CONFIGCHANGECANCEL");
						break;
				}

				// Now that we have reported the 'class' of message that
				// has come through via the WM_DEVICECHANGE message,
				// Lets look at any additional information provided
				// in the lParam, and display that to the user.
				switch (uEvent) {
					// For devices that use DEV_BROADCAST_HDR or derivaties:
					case DBT_DEVICEARRIVAL:
					case DBT_DEVICEQUERYREMOVE:
					case DBT_DEVICEQUERYREMOVEFAILED:
					case DBT_DEVICEREMOVEPENDING:
					case DBT_DEVICEREMOVECOMPLETE:
					case DBT_DEVICETYPESPECIFIC:
					case DBT_CONFIGCHANGED:
						pDBH = (DEV_BROADCAST_HDR*)dwData;
						AppendEditString (hwndOutput, " : ");
						switch (pDBH->dbch_devicetype) {

							case DBT_DEVTYP_OEM:	//OEM- or IHV-defined device type.
								AppendEditString (hwndOutput, "DBT_DEVTYP_OEM");
								wsprintf (sz, " [%d=ID, %d=SuppFunc]",
									((DEV_BROADCAST_OEM*)pDBH)->dbco_identifier,
									((DEV_BROADCAST_OEM*)pDBH)->dbco_suppfunc);
								AppendEditString (hwndOutput, sz);
								break;

							case DBT_DEVTYP_VOLUME:	//Logical volume.
								AppendEditString (hwndOutput, "DBT_DEVTYP_VOLUME");

								if (((DEV_BROADCAST_VOLUME*)pDBH)->dbcv_flags & DBTF_MEDIA) {
									AppendEditString (hwndOutput, " : DBTF_MEDIA");
								} else if (((DEV_BROADCAST_VOLUME*)pDBH)->dbcv_flags & DBTF_NET) {
									AppendEditString (hwndOutput, " : DBTF_NET");
								} else {
									wsprintf (sz, " : 0x%X = Flags", ((DEV_BROADCAST_VOLUME*)pDBH)->dbcv_flags);
									AppendEditString (hwndOutput, sz);
								}

								{
									UINT unitmask = ((DEV_BROADCAST_VOLUME*)pDBH)->dbcv_unitmask;
									char drive[2] = "A";
									AppendEditString (hwndOutput, " [");
									for (drive[0]='A'; unitmask!=0; drive[0]++, unitmask=unitmask>>1) {
										if (unitmask & 1) {
											AppendEditString (hwndOutput, drive);
											AppendEditString (hwndOutput,": ");
										}
									}
									AppendEditString (hwndOutput, "] ");
								}
								break;

							case DBT_DEVTYP_PORT:	//Port device (serial or parallel).
								AppendEditString (hwndOutput, "DBT_DEVTYP_PORT");
								AppendEditString (hwndOutput, " : Name=");
								AppendEditString (hwndOutput, ((DEV_BROADCAST_PORT*)pDBH)->dbcp_name);
								break;
						}
						break;

					case DBT_MONITORCHANGE:
						if (lParam) {
							wsprintf (sz, " : (%d, %d)", LOWORD(lParam), HIWORD(lParam));
							AppendEditString (hwndOutput, sz);
						} else {
							AppendEditString (hwndOutput, " 0 = Use defaults for current config");
						}
						break;

					case DBT_USERDEFINED:
						pDBU = (struct _DEV_BROADCAST_USERDEFINED *)lParam;
						AppendEditString (hwndOutput, " - Name:");
						AppendEditString (hwndOutput, pDBU->dbud_szName);
						wsprintf (sz, " [%d additional bytes of data]",
							pDBU->dbud_dbh.dbch_size - sizeof(struct _DEV_BROADCAST_USERDEFINED));
						AppendEditString (hwndOutput, sz);
						break;

					// The following make no use of lParam:
					case DBT_CONFIGCHANGECANCELED:
						break;
				}

			}
			break;

		case WM_POWERBROADCAST:
		// The WM_POWERBROADCAST message is sent to an application to
		// notify it of power-management events.
		// Windows NT: For Windows NT applications, use WM_POWER.
			AppendEditString (hwndOutput, "WM_POWERBROADCAST");
			switch (wParam) {
				case PBT_APMPOWERSTATUSCHANGE:	// Power status has changed.
					AppendEditString (hwndOutput, " : PB_APMPOWERSTATUSCHANGED");
					wsprintf (sz, " : 0x%X", lParam);
					AppendEditString (hwndOutput, sz);
					{
						SYSTEM_POWER_STATUS sps;
						if (GetSystemPowerStatus (&sps)) {
							AppendEditString (hwndOutput, "\n");
							AppendEditString (hwndOutput, "\tACLineStatus = ");
							if (sps.ACLineStatus==0){
								AppendEditString (hwndOutput, "OffLine");
							} else if (sps.ACLineStatus==1) {
								AppendEditString (hwndOutput, "OnLine");
							} else if (sps.ACLineStatus==255) {
								AppendEditString (hwndOutput, "Unknown Status");
							} else {
								AppendEditString (hwndOutput, "Unexpected Status");
							}
							AppendEditString (hwndOutput, " : Battery = ");
							switch (sps.BatteryFlag) {
								case 1: 
									AppendEditString (hwndOutput, "High");
									break;
								case 2:
									AppendEditString (hwndOutput, "Low");
									break;
								case 4:
									AppendEditString (hwndOutput, "Critical");
									break;
								case 8:
									AppendEditString (hwndOutput, "Charging");
									break;
								case 128:
									AppendEditString (hwndOutput, "No system battery");
									break;
								case 255:
									AppendEditString (hwndOutput, "Unknown status");
									break;
								default:
									AppendEditString (hwndOutput, "Unexpected status");
									break;
							}

							AppendEditString (hwndOutput, " : BatteryLifePercent = ");
							if (sps.BatteryLifePercent == 255) {
								AppendEditString (hwndOutput, "Unknown");
							} else {
								wsprintf (sz, "%d", sps.BatteryLifePercent);
								AppendEditString (hwndOutput, sz);
							}

							AppendEditString (hwndOutput, " : BatteryLifeTime = ");
							if (sps.BatteryLifeTime == -1) {
								AppendEditString (hwndOutput, "Unknown");
							} else {
								wsprintf (sz, "%d", sps.BatteryLifeTime);
								AppendEditString (hwndOutput, sz);
							}

							AppendEditString (hwndOutput, " : BatteryFullLifeTime = ");
							if (sps.BatteryFullLifeTime == -1) {
								AppendEditString (hwndOutput, "Unknown");
							} else {
								wsprintf (sz, "%d", sps.BatteryFullLifeTime);
								AppendEditString (hwndOutput, sz);
							}

						}
					}
					break;

				case PBT_APMQUERYSTANDBY:	// Request for permission to standby.
					AppendEditString (hwndOutput, "PB_APMQUERYSTANDBY : ");
					AppendEditString (hwndOutput, (lParam&1)?"Allow User Interaction":"Don't Prompt User");
					break;
				case PBT_APMQUERYSTANDBYFAILED:	// Standby request denied.
					AppendEditString (hwndOutput, "PB_APMSTANDBYFAILED");
					break;
				case PBT_APMSTANDBY:	// System is changing to standby.
					AppendEditString (hwndOutput, "PB_APMSTANDBY");
					break;
				case PBT_APMRESUMESTANDBY:	// Operation resuming after standby.
					AppendEditString (hwndOutput, "PB_APMRESUMESTANDBY");
					if (lParam&PBTF_APMRESUMEFROMFAILURE) {
						AppendEditString (hwndOutput, " : PB_APMRESUMEFROMFAILURE");
					}
					break;

				case PBT_APMQUERYSUSPEND:	// Request for permission to suspend.
					AppendEditString (hwndOutput, "PB_APMQUERYSUSPEND : ");
					AppendEditString (hwndOutput, (lParam&1)?"Allow User Interaction":"Don't Prompt User");
					break;
				case PBT_APMQUERYSUSPENDFAILED:	// Suspension request denied.
					AppendEditString (hwndOutput, "PB_APMQUERYSUSPENDFAILED");
					break;
				case PBT_APMSUSPEND:	// System is suspending operation.
					AppendEditString (hwndOutput, "PB_APMSUSPEND");
					break;
				case PBT_APMRESUMESUSPEND:	// Operation resuming after suspension.
					AppendEditString (hwndOutput, "PB_APMRESUMESUSPEND");
					if (lParam&PBTF_APMRESUMEFROMFAILURE) {
						AppendEditString (hwndOutput, " : PB_APMRESUMEFROMFAILURE");
					}
					break;

				case PBT_APMRESUMECRITICAL:	// Operation resuming after critical suspension.
					AppendEditString (hwndOutput, "PB_APMRESUMECRITICAL");
					if (lParam&PBTF_APMRESUMEFROMFAILURE) {
						AppendEditString (hwndOutput, " : PB_APMRESUMEFROMFAILURE");
					}
					break;

				case PBT_APMBATTERYLOW:
					AppendEditString (hwndOutput, "PB_APMBATTERYLOW");
					wsprintf (sz, " : lParam = %d", lParam);
					AppendEditString (hwndOutput, sz);
					break;

				case PBT_APMOEMEVENT:
					AppendEditString (hwndOutput, "PB_APMOEMEVENT");
					wsprintf (sz, " : BIOS Event = 0x%X", lParam);
					AppendEditString (hwndOutput, sz);
					break;

				default:
					wsprintf (sz, " : wParam = 0x%X, lParam = 0x%X", wParam, lParam);
					AppendEditString (hwndOutput, sz);
					break;
			}
			break;

		case WM_DISPLAYCHANGE:
		// The WM_DISPLAYCHANGE message is sent to all windows when
		// the display resolution has changed.
			AppendEditString (hwndOutput, "WM_DISPLAYCHANGE");
			{
				WORD cBitsPerPixel = (BOOL)wParam;
				WORD cxChange = LOWORD(lParam);
				WORD cyChange  = HIWORD(lParam);
				wsprintf (sz, "%(d x %d) BitsPerPixsl = %d", cxChange, cyChange, cBitsPerPixel);
				AppendEditString (hwndOutput, sz);
			}
			break;

		case WM_SETTINGCHANGE:
		// The WM_SETTINGCHANGE message is sent to all windows when an
		// application changes a systemwide setting by using the
		// SystemParametersInfo function.
			AppendEditString (hwndOutput, "WM_SETTINGGHANGE");
			wsprintf (sz, " : wFlag = %d, pszMetrics = %s", wParam,
				(BOOL)lParam?(LPSTR)lParam:"-not appropriate-");
			AppendEditString (hwndOutput, sz);
			break;

		case WM_INPUTLANGCHANGEREQUEST:
		// The WM_INPUTLANGCHANGEREQUEST message is sent when the user chooses
		// an input language, either with an input language change hotkey or
		// from the system languages menu. An application can accept the change
		// by passing the message to the DefWindowProc function or reject the
		// change (and prevent it from taking place) by returning immediately.
			AppendEditString (hwndOutput, "WM_INPUTLANGCHANGEREQUEST");
			wsprintf (sz, " : fSysCharSet = %s, HKL = 0x%X", (BOOL)wParam?"TRUE":"FALSE", lParam);
			AppendEditString (hwndOutput, sz);
			break;

		case WM_INPUTLANGCHANGE:
		// The WM_INPUTLANGCHANGE message is sent to the topmost affected window
		// after a task's locale has been changed. It should be used to make any
		// application-specific settings and passed on to the DefWindowProc
		// function to be passed on to any children.
			AppendEditString (hwndOutput, "WM_INPUTLANGCHANGE");
			wsprintf (sz, " : charset = %d, HKL = 0x%X : ", wParam, lParam);
			AppendEditString (hwndOutput, sz);
			if (GetKeyboardLayoutName (sz)) {
				AppendEditString (hwndOutput, sz);
			}
			break;

		case WM_USERCHANGED:
		// The WM_USERCHANGED message is sent to all windows after the user has
		// logged on or off. When the user logs on or off, the system updates the
		// user-specific settings. The system sends this message immediately after
		// updating the settings. Both wParam and lParam should be '0'.
		// Note: Applications will probably never get this, since they are
		// shut down when a user exits, and any that are started back up, are
		// done so after the user is fully logged in.
			AppendEditString (hwndOutput, "WM_USERCHANGE");
			wsprintf (sz, " : wParam = %d, lParam = %d", wParam, lParam);
			AppendEditString (hwndOutput, sz);
			break;

		case WM_POWER:
		// The WM_POWER message is sent when the system, typically a
		// battery-powered personal computer, is about to enter suspended mode. 
		// Windows 95: For Windows 95 applications, The WM_POWER message is not
		// supported. It is provided to simplify porting of 16-bit Windows-based
		// applications. Win32-based applications for Windows 95 should use the
		// WM_POWERBROADCAST message.
			AppendEditString (hwndOutput, "WM_POWER");
			{
				WPARAM fwPowerEvt = wParam;
				switch (fwPowerEvt) {
					case PWR_CRITICALRESUME:
						AppendEditString (hwndOutput, " : PWR_CRITICALRESUME");
						break;
					case PWR_SUSPENDREQUEST:
						AppendEditString (hwndOutput, ": PWR_SUSPENDREQUEST");
						break;
					case PWR_SUSPENDRESUME:
						AppendEditString (hwndOutput, ": PWR_SUSPENDRESUME");
						break;
				}
			}
			break;

		case WM_DEVMODECHANGE:
		// The WM_DEVMODECHANGE message is sent to all top-level windows
		// whenever the user changes device-mode settings.
			AppendEditString (hwndOutput, "WM_DEVMODECHANGE : ");
			AppendEditString (hwndOutput, (LPSTR)lParam);
			break;

		// And lets not forget:
		case WM_QUERYENDSESSION:
			AppendEditString (hwndOutput, "WM_QUERYENDSESSION");
			wsprintf (sz, " : %d = %s", (BOOL)wParam?"Via TaskList":"Via Logoff/Shutdown");
			AppendEditString (hwndOutput, sz);
			break;

		case WM_ENDSESSION:
			AppendEditString (hwndOutput, "WM_ENDSESSION");
			wsprintf (" : %s", (BOOL)wParam?"Going Down":"Not Going Down");
			AppendEditString (hwndOutput, sz);
			break;

		default:
			wsprintf (sz, "- Unexpected Message : 0x%X -", message);
			AppendEditString (hwndOutput, sz);
			break;
	}

	AppendEditString (hwndOutput, "\n");
	return 0;
}
